/*
  Lab 8 Skeleton 
  Chris Szilagyi
  10-12-2009
  lab_8.c
  
  This program uses the lcd_lib and modclock libraries.  They must be
  included in the project.

  
  Wiring:
  Wire Port B to the on-board LED's according the following list:
  PT0 - LED1
  PT1 - LED2
  PT2 - LED3
  PT3 - LED4
  PT4 - LED5
  PT5 - LED6
  PT6 - LED7
  PT7 - LED8
  
  Additionally wire according to Lab #3 (Wiper Controller).
*/


#include <hidef.h>      /* common defines and macros */
#include <mc9s12c128.h>     /* derivative information */
#pragma LINK_INFO DERIVATIVE "mc9s12c128"

//other includes
#include <stdio.h>
#include "lcd_lib.h"
#include "modclock.h"
#include "main_asm.h" /* interface to the assembly module */

/*****************
 *    typedefs
 *****************/
typedef char bool;
#define true 1
#define false 0

/***************************
 *    function prototypes
 ***************************/
void setupTimer( void );
void setupSerial( void );


/*************************
 *    global variables
 *************************/

//timer globals
char timerBuf[9];
unsigned long timerCount;
unsigned int oldSeconds;

//serial globals
char sciRxBuf[8];

/******************
 *    defines
 ******************/

//define the various indicator bits to indicate status on port T.
#define TIMER_INDICATOR 0x80 //indicates that a timer overflow ISR is in progress
#define RDRF_INDICATOR 0x40 //indicates that a UART receive ISR is in progress
#define OR_INDICATOR 0x20  //indicates that a UART receive overflow ISR is in progress
#define LCD_WRITE_INDICATOR 0x10 //indicates that the LCD_WRITE activity is in progress
#define MAIN_LOOP_INDICATOR 0x01 //this bit is toggled at the completion of the main
                                 //loop, so a cycle (transition to low and back to high) 
                                 //actually represents 2 main loop cycles.

//define PTB_STATUS to enable PORTB outputs
#define PTB_STATUS  

//define BLOCK_DURING_LCD to cause the main loop to 
//block interrupts during the LCD write
//#define BLOCK_DURING_LCD





/******************************
 *        Main Function
 ******************************/
void main(void) {
  unsigned char chaser;
#ifdef PTB_STATUS  
  unsigned char ptbVal;
#endif


  DDRB = 0xFF;
  PORTB = 0x00;

  //setup tasks
  clockSetup();
  lcdSetup();
  setupTimer();
  setupSerial();
  
  //main loop init
  oldSeconds = 0xFFFF;
  chaser = 0x1;


  wiper_init();

  EnableInterrupts;

  //main task loop - this gets executed forever
  for(;;) {

    /*****************
      Begin the main loop task
     *****************/

#ifdef BLOCK_DURING_LCD
    DisableInterrupts;
#endif
#ifdef PTB_STATUS  
    PORTB |= LCD_WRITE_INDICATOR;    
#endif
    lcdWriteLine(2,timerBuf);
    lcdWriteLine(1,sciRxBuf);
#ifdef PTB_STATUS
    PORTB &= ~LCD_WRITE_INDICATOR;
#endif
#ifdef BLOCK_DURING_LCD
    EnableInterrupts;
#endif
    
      wiper_main();


#ifdef PTB_STATUS  
    ptbVal = PORTB;
    PORTB = (ptbVal & ~MAIN_LOOP_INDICATOR) | (~ptbVal & MAIN_LOOP_INDICATOR);
#endif    

    /*****************
      End the main loop task
     *****************/
    

  }
  /* please make sure that you never leave this function */
}


/*********************
 *     Timer Task
 *********************/
//setup for timer tasks
void setupTimer( void ) {

/*TODO:  Setup timer */
  
	/* timer will operate at 0.5 MHz, enable TOI interrupt*/
	/* timer will stop in freeze and wait modes */
	/* initilize global var timerCount to 0 */
  /* initialize the timer string to "00:00:00"*/

}

//setup for serial tasks
void setupSerial( void ) {

/*TODO: Setup UART */

/*38400 baud, 8 bit, one stop bit, no parity*/
/*Enable receiver, and RIE*/
/*initialize sciRxBuf to all spaces*/


}


/*****************************************
 *    Interrupt Service Routines
 *****************************************/
 
/*
  timer ISR
  
  Increment the timer counter and update the display
  when necessary.
*/
void interrupt 16 timerHandler(void) { 
  unsigned int wholeSeconds;
  unsigned char seconds;
  unsigned char minutes;
  unsigned int hours;

#ifdef PTB_STATUS  
  PORTB |= TIMER_INDICATOR;
#endif

  /*increment timerCount*/
  /*acknowledge interrupt*/

  /*update the display value if the # of seconds has changed*/
  /*NOTE: ALL code for this function must appear between the BEGIN_timer and END_timer comments*/

/*BEGIN_timer*/




/*END_timer*/

#ifdef PTB_STATUS  
  PORTB &= ~TIMER_INDICATOR;
#endif
}

